(***********************************************************************)
(*                                                                     *)
(*                           Htmlc                                     *)
(*                                                                     *)
(*                    Pierre Weis, INRIA Rocquencourt                  *)
(*                                                                     *)
(*  Copyright 1997-2007,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: htmlc_types.mli,v 1.1 2015/11/18 18:07:23 weis Exp $ *)

(** {3 Definition and operations on the Htmlc environment.} *)

type ident = string
(** Variables environment to expand into files. *)
;;

type command_name = string
(** The name of an operating system command. *)
;;

type htmlc_definitions =
   | Htmlc_environment of File.file_name
   | Htmlc_bindings of File.file_name
   | Htmlc_definition of ident * string
(** The kind of definitions that could be added to the Htmlc runtime
    environment. *)
;;

type htmlc_mode =
   | Mode_verbatim
   | Mode_source
   | Mode_ssi
   | Mode_non_ssi
(** The pre-processing modes for Htmlc:
    - [Mode_verbatim] means no pre-processing.
    - [Mode_source] means pre-processing using the pre-processors for source
    files.
    - [Mode_ssi] means pre-processing using the pre-processors for file
    inclusion from a SSI directive.
    - [Mode_non_ssi] means pre-processing using the pre-processors for file
    inclusion from a non-SSI directive. *)
;;

type line_continuation_mode =
  | Keep_verbatim
  | Skip of leading_spaces_mode

and leading_spaces_mode =
  | Keep_leading_spaces
  | Skip_leading_spaces
(** The line continuation treatment modes for Htmlc:
    - [Keep_verbatim] means do not process line continuation.
    - [Skip leading_spaces_mode] means process line continuation
      using one of:
      - [Keep_leading_spaces], leading spaces are kept in the output (the
        shell way of treating line continuations),
      - [Skip_leading_spaces], skip leading spaces are skipped from the
        output (the Caml way of treating line continuations). *)
;;
