(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2005-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: options_pio.ml,v 1.31 2017/05/31 15:54:22 fclement Exp $ *)

(* Report/debug flags and warning/error messages. *)

module Ar = Uaref;;
module Ok = Uoption_key;;
module Co = Ucommand_option;;
module D = Udebug;;

module Vp = Version_pio;;

let software_name = Vp.option_key_data.Co.name;;

let pf = D.library_printing_facilities Vp.option_key_data;;

let state_of_bool = function
  | true -> "on"
  | false -> "off"
;;

(* Binary/ASCII modes. *)

let ({ Ar.get_tflag = get_binary_mode; toggle_tflag = set_binary_mode; _ },
     { Ar.get_tflag = get_ascii_mode; toggle_tflag = set_ascii_mode; _ }) =
  Ar.make_antonymic_toggle_flags true;;

let gen_mode_lok mode =
  Ok.long_option_key_of_string (Printf.sprintf "%s %s" software_name mode)
;;

let ascii_mode_lok = gen_mode_lok "ascii"
and binary_mode_lok = gen_mode_lok "binary"
;;

let ascii_mode_rco =
  {
   Co.key = ascii_mode_lok;
   spec = Arg.Unit set_ascii_mode;
   doc =
     Printf.sprintf
       "\n\
       \tTurn %s ascii mode %s.\n\
       \tFloating-point numbers are written using an ascii \
         human-readable format.\n\
       \tThis mode is exclusive to binary mode. Default is %s."
       software_name (state_of_bool true)
       (state_of_bool (get_ascii_mode ()));
  }

and binary_mode_rco =
  {
   Co.key = binary_mode_lok;
   spec = Arg.Unit set_binary_mode;
   doc =
     Printf.sprintf
       "\n\
       \tTurn %s binary mode %s.\n\
       \tFloating-point numbers are written using a compact binary \
         format.\n\
       \tThis mode is exclusive to ascii mode. Default is %s."
       software_name (state_of_bool true)
       (state_of_bool (get_binary_mode ()));
  }
;;

(* Command-line options. *)

let add_options () =
  Co.library_add_all_rco_auto
    [
     ascii_mode_rco;
     binary_mode_rco;
    ]
;;

add_options ();;
