(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright (c) 2005-2015 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* Beginning of OCamlGen, 4.0+dev0 generated code. *)

let printer_native_int ctx ppf = function
  | n_t -> Nativeint_print.printer_t ctx ppf n_t

and printer_int32 ctx ppf = function
  | i_t -> Int32_print.printer_t ctx ppf i_t

and printer_int64 ctx ppf = function
  | i_t -> Int64_print.printer_t ctx ppf i_t
;;

let rec printer_t ctx ppf = function
  | Typed_value.Scalar s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Scalar";
    printer_scalar Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vector v ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vector";
    printer_vector Og_precedence.Ck_argument ppf v;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Matrix m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Matrix";
    printer_matrix Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_scalar ctx ppf = function
  | Typed_value.Sbool b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Sbool";
    Og_printer.printer_Bool Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Sbool_couple (b, b0) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Sbool_couple";
    Format.fprintf ppf "%, (@,"; Format.fprintf ppf "%,@[<hv 0>";
    Og_printer.printer_Bool Og_precedence.Ck_tuple_item ppf b;
    Format.fprintf ppf "%,,@ ";
    Og_printer.printer_Bool Og_precedence.Ck_tuple_item ppf b0;
    Format.fprintf ppf "%,@]"; Format.fprintf ppf "%,@;<0 -2>)";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Sbool_triple (b, b0, b1) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Sbool_triple";
    Format.fprintf ppf "%, (@,"; Format.fprintf ppf "%,@[<hv 0>";
    Og_printer.printer_Bool Og_precedence.Ck_tuple_item ppf b;
    Format.fprintf ppf "%,,@ ";
    Og_printer.printer_Bool Og_precedence.Ck_tuple_item ppf b0;
    Format.fprintf ppf "%,,@ ";
    Og_printer.printer_Bool Og_precedence.Ck_tuple_item ppf b1;
    Format.fprintf ppf "%,@]"; Format.fprintf ppf "%,@;<0 -2>)";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Sstring s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Sstring";
    Og_printer.printer_String Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Sint i ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Sint";
    Og_printer.printer_Int Og_precedence.Ck_argument ppf i;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Sint_couple (i, i0) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Sint_couple";
    Format.fprintf ppf "%, (@,"; Format.fprintf ppf "%,@[<hv 0>";
    Og_printer.printer_Int Og_precedence.Ck_tuple_item ppf i;
    Format.fprintf ppf "%,,@ ";
    Og_printer.printer_Int Og_precedence.Ck_tuple_item ppf i0;
    Format.fprintf ppf "%,@]"; Format.fprintf ppf "%,@;<0 -2>)";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Sint_triple (i, i0, i1) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Sint_triple";
    Format.fprintf ppf "%, (@,"; Format.fprintf ppf "%,@[<hv 0>";
    Og_printer.printer_Int Og_precedence.Ck_tuple_item ppf i;
    Format.fprintf ppf "%,,@ ";
    Og_printer.printer_Int Og_precedence.Ck_tuple_item ppf i0;
    Format.fprintf ppf "%,,@ ";
    Og_printer.printer_Int Og_precedence.Ck_tuple_item ppf i1;
    Format.fprintf ppf "%,@]"; Format.fprintf ppf "%,@;<0 -2>)";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Snative_int n ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Snative_int";
    printer_native_int Og_precedence.Ck_argument ppf n;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Snative_int_couple (n, n0) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Snative_int_couple";
    Format.fprintf ppf "%, (@,"; Format.fprintf ppf "%,@[<hv 0>";
    printer_native_int Og_precedence.Ck_tuple_item ppf n;
    Format.fprintf ppf "%,,@ ";
    printer_native_int Og_precedence.Ck_tuple_item ppf n0;
    Format.fprintf ppf "%,@]"; Format.fprintf ppf "%,@;<0 -2>)";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Snative_int_triple (n, n0, n1) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Snative_int_triple";
    Format.fprintf ppf "%, (@,"; Format.fprintf ppf "%,@[<hv 0>";
    printer_native_int Og_precedence.Ck_tuple_item ppf n;
    Format.fprintf ppf "%,,@ ";
    printer_native_int Og_precedence.Ck_tuple_item ppf n0;
    Format.fprintf ppf "%,,@ ";
    printer_native_int Og_precedence.Ck_tuple_item ppf n1;
    Format.fprintf ppf "%,@]"; Format.fprintf ppf "%,@;<0 -2>)";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Sint32 i ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Sint32";
    printer_int32 Og_precedence.Ck_argument ppf i; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Sint32_couple (i, i0) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Sint32_couple";
    Format.fprintf ppf "%, (@,"; Format.fprintf ppf "%,@[<hv 0>";
    printer_int32 Og_precedence.Ck_tuple_item ppf i;
    Format.fprintf ppf "%,,@ ";
    printer_int32 Og_precedence.Ck_tuple_item ppf i0;
    Format.fprintf ppf "%,@]"; Format.fprintf ppf "%,@;<0 -2>)";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Sint32_triple (i, i0, i1) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Sint32_triple";
    Format.fprintf ppf "%, (@,"; Format.fprintf ppf "%,@[<hv 0>";
    printer_int32 Og_precedence.Ck_tuple_item ppf i;
    Format.fprintf ppf "%,,@ ";
    printer_int32 Og_precedence.Ck_tuple_item ppf i0;
    Format.fprintf ppf "%,,@ ";
    printer_int32 Og_precedence.Ck_tuple_item ppf i1;
    Format.fprintf ppf "%,@]"; Format.fprintf ppf "%,@;<0 -2>)";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Sint64 i ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Sint64";
    printer_int64 Og_precedence.Ck_argument ppf i; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Sint64_couple (i, i0) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Sint64_couple";
    Format.fprintf ppf "%, (@,"; Format.fprintf ppf "%,@[<hv 0>";
    printer_int64 Og_precedence.Ck_tuple_item ppf i;
    Format.fprintf ppf "%,,@ ";
    printer_int64 Og_precedence.Ck_tuple_item ppf i0;
    Format.fprintf ppf "%,@]"; Format.fprintf ppf "%,@;<0 -2>)";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Sint64_triple (i, i0, i1) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Sint64_triple";
    Format.fprintf ppf "%, (@,"; Format.fprintf ppf "%,@[<hv 0>";
    printer_int64 Og_precedence.Ck_tuple_item ppf i;
    Format.fprintf ppf "%,,@ ";
    printer_int64 Og_precedence.Ck_tuple_item ppf i0;
    Format.fprintf ppf "%,,@ ";
    printer_int64 Og_precedence.Ck_tuple_item ppf i1;
    Format.fprintf ppf "%,@]"; Format.fprintf ppf "%,@;<0 -2>)";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Sfloat f ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Sfloat";
    Og_printer.printer_Float Og_precedence.Ck_argument ppf f;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Sfloat_couple (f, f0) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Sfloat_couple";
    Format.fprintf ppf "%, (@,"; Format.fprintf ppf "%,@[<hv 0>";
    Og_printer.printer_Float Og_precedence.Ck_tuple_item ppf f;
    Format.fprintf ppf "%,,@ ";
    Og_printer.printer_Float Og_precedence.Ck_tuple_item ppf f0;
    Format.fprintf ppf "%,@]"; Format.fprintf ppf "%,@;<0 -2>)";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Sfloat_triple (f, f0, f1) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Sfloat_triple";
    Format.fprintf ppf "%, (@,"; Format.fprintf ppf "%,@[<hv 0>";
    Og_printer.printer_Float Og_precedence.Ck_tuple_item ppf f;
    Format.fprintf ppf "%,,@ ";
    Og_printer.printer_Float Og_precedence.Ck_tuple_item ppf f0;
    Format.fprintf ppf "%,,@ ";
    Og_printer.printer_Float Og_precedence.Ck_tuple_item ppf f1;
    Format.fprintf ppf "%,@]"; Format.fprintf ppf "%,@;<0 -2>)";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_vector ctx ppf = function
  | Typed_value.Vbool b_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vbool";
    Og_printer.printer_Array Og_printer.printer_Bool
      Og_precedence.Ck_argument
      ppf
      b_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vbool_couple t_b_b_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vbool_couple";
    Og_printer.printer_Array
      (fun ctx ppf (b, b0) ->
       if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
         else
       if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@,"
         else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@[<hv 2>(@,";
       Format.fprintf ppf "%,@[<hv 0>";
       Og_printer.printer_Bool Og_precedence.Ck_tuple_item ppf b;
       Format.fprintf ppf "%,,@ ";
       Og_printer.printer_Bool Og_precedence.Ck_tuple_item ppf b0;
       Format.fprintf ppf "%,@]";
       if ctx = Og_precedence.Ck_argument then
         Format.fprintf ppf "%,@;<0 -2>)" else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@;<0 -2>)@]")
      Og_precedence.Ck_argument
      ppf
      t_b_b_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vbool_triple t_b_b_b_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vbool_triple";
    Og_printer.printer_Array
      (fun ctx ppf (b, b0, b1) ->
       if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
         else
       if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@,"
         else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@[<hv 2>(@,";
       Format.fprintf ppf "%,@[<hv 0>";
       Og_printer.printer_Bool Og_precedence.Ck_tuple_item ppf b;
       Format.fprintf ppf "%,,@ ";
       Og_printer.printer_Bool Og_precedence.Ck_tuple_item ppf b0;
       Format.fprintf ppf "%,,@ ";
       Og_printer.printer_Bool Og_precedence.Ck_tuple_item ppf b1;
       Format.fprintf ppf "%,@]";
       if ctx = Og_precedence.Ck_argument then
         Format.fprintf ppf "%,@;<0 -2>)" else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@;<0 -2>)@]")
      Og_precedence.Ck_argument
      ppf
      t_b_b_b_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vstring s_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vstring";
    Og_printer.printer_Array Og_printer.printer_String
      Og_precedence.Ck_argument
      ppf
      s_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vint i_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vint";
    Og_printer.printer_Array Og_printer.printer_Int
      Og_precedence.Ck_argument
      ppf
      i_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vint_couple t_i_i_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vint_couple";
    Og_printer.printer_Array
      (fun ctx ppf (i, i0) ->
       if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
         else
       if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@,"
         else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@[<hv 2>(@,";
       Format.fprintf ppf "%,@[<hv 0>";
       Og_printer.printer_Int Og_precedence.Ck_tuple_item ppf i;
       Format.fprintf ppf "%,,@ ";
       Og_printer.printer_Int Og_precedence.Ck_tuple_item ppf i0;
       Format.fprintf ppf "%,@]";
       if ctx = Og_precedence.Ck_argument then
         Format.fprintf ppf "%,@;<0 -2>)" else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@;<0 -2>)@]")
      Og_precedence.Ck_argument
      ppf
      t_i_i_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vint_triple t_i_i_i_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vint_triple";
    Og_printer.printer_Array
      (fun ctx ppf (i, i0, i1) ->
       if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
         else
       if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@,"
         else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@[<hv 2>(@,";
       Format.fprintf ppf "%,@[<hv 0>";
       Og_printer.printer_Int Og_precedence.Ck_tuple_item ppf i;
       Format.fprintf ppf "%,,@ ";
       Og_printer.printer_Int Og_precedence.Ck_tuple_item ppf i0;
       Format.fprintf ppf "%,,@ ";
       Og_printer.printer_Int Og_precedence.Ck_tuple_item ppf i1;
       Format.fprintf ppf "%,@]";
       if ctx = Og_precedence.Ck_argument then
         Format.fprintf ppf "%,@;<0 -2>)" else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@;<0 -2>)@]")
      Og_precedence.Ck_argument
      ppf
      t_i_i_i_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vnative_int n_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vnative_int";
    Og_printer.printer_Array printer_native_int
      Og_precedence.Ck_argument
      ppf
      n_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vnative_int_couple t_n_n_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vnative_int_couple";
    Og_printer.printer_Array
      (fun ctx ppf (n, n0) ->
       if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
         else
       if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@,"
         else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@[<hv 2>(@,";
       Format.fprintf ppf "%,@[<hv 0>";
       printer_native_int Og_precedence.Ck_tuple_item ppf n;
       Format.fprintf ppf "%,,@ ";
       printer_native_int Og_precedence.Ck_tuple_item ppf n0;
       Format.fprintf ppf "%,@]";
       if ctx = Og_precedence.Ck_argument then
         Format.fprintf ppf "%,@;<0 -2>)" else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@;<0 -2>)@]")
      Og_precedence.Ck_argument
      ppf
      t_n_n_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vnative_int_triple t_n_n_n_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vnative_int_triple";
    Og_printer.printer_Array
      (fun ctx ppf (n, n0, n1) ->
       if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
         else
       if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@,"
         else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@[<hv 2>(@,";
       Format.fprintf ppf "%,@[<hv 0>";
       printer_native_int Og_precedence.Ck_tuple_item ppf n;
       Format.fprintf ppf "%,,@ ";
       printer_native_int Og_precedence.Ck_tuple_item ppf n0;
       Format.fprintf ppf "%,,@ ";
       printer_native_int Og_precedence.Ck_tuple_item ppf n1;
       Format.fprintf ppf "%,@]";
       if ctx = Og_precedence.Ck_argument then
         Format.fprintf ppf "%,@;<0 -2>)" else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@;<0 -2>)@]")
      Og_precedence.Ck_argument
      ppf
      t_n_n_n_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vint32 i_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vint32";
    Og_printer.printer_Array printer_int32 Og_precedence.Ck_argument ppf i_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vint32_couple t_i_i_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vint32_couple";
    Og_printer.printer_Array
      (fun ctx ppf (i, i0) ->
       if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
         else
       if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@,"
         else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@[<hv 2>(@,";
       Format.fprintf ppf "%,@[<hv 0>";
       printer_int32 Og_precedence.Ck_tuple_item ppf i;
       Format.fprintf ppf "%,,@ ";
       printer_int32 Og_precedence.Ck_tuple_item ppf i0;
       Format.fprintf ppf "%,@]";
       if ctx = Og_precedence.Ck_argument then
         Format.fprintf ppf "%,@;<0 -2>)" else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@;<0 -2>)@]")
      Og_precedence.Ck_argument
      ppf
      t_i_i_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vint32_triple t_i_i_i_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vint32_triple";
    Og_printer.printer_Array
      (fun ctx ppf (i, i0, i1) ->
       if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
         else
       if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@,"
         else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@[<hv 2>(@,";
       Format.fprintf ppf "%,@[<hv 0>";
       printer_int32 Og_precedence.Ck_tuple_item ppf i;
       Format.fprintf ppf "%,,@ ";
       printer_int32 Og_precedence.Ck_tuple_item ppf i0;
       Format.fprintf ppf "%,,@ ";
       printer_int32 Og_precedence.Ck_tuple_item ppf i1;
       Format.fprintf ppf "%,@]";
       if ctx = Og_precedence.Ck_argument then
         Format.fprintf ppf "%,@;<0 -2>)" else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@;<0 -2>)@]")
      Og_precedence.Ck_argument
      ppf
      t_i_i_i_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vint64 i_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vint64";
    Og_printer.printer_Array printer_int64 Og_precedence.Ck_argument ppf i_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vint64_couple t_i_i_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vint64_couple";
    Og_printer.printer_Array
      (fun ctx ppf (i, i0) ->
       if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
         else
       if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@,"
         else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@[<hv 2>(@,";
       Format.fprintf ppf "%,@[<hv 0>";
       printer_int64 Og_precedence.Ck_tuple_item ppf i;
       Format.fprintf ppf "%,,@ ";
       printer_int64 Og_precedence.Ck_tuple_item ppf i0;
       Format.fprintf ppf "%,@]";
       if ctx = Og_precedence.Ck_argument then
         Format.fprintf ppf "%,@;<0 -2>)" else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@;<0 -2>)@]")
      Og_precedence.Ck_argument
      ppf
      t_i_i_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vint64_triple t_i_i_i_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vint64_triple";
    Og_printer.printer_Array
      (fun ctx ppf (i, i0, i1) ->
       if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
         else
       if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@,"
         else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@[<hv 2>(@,";
       Format.fprintf ppf "%,@[<hv 0>";
       printer_int64 Og_precedence.Ck_tuple_item ppf i;
       Format.fprintf ppf "%,,@ ";
       printer_int64 Og_precedence.Ck_tuple_item ppf i0;
       Format.fprintf ppf "%,,@ ";
       printer_int64 Og_precedence.Ck_tuple_item ppf i1;
       Format.fprintf ppf "%,@]";
       if ctx = Og_precedence.Ck_argument then
         Format.fprintf ppf "%,@;<0 -2>)" else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@;<0 -2>)@]")
      Og_precedence.Ck_argument
      ppf
      t_i_i_i_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vfloat f_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vfloat";
    Og_printer.printer_Array Og_printer.printer_Float
      Og_precedence.Ck_argument
      ppf
      f_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vfloat_couple t_f_f_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vfloat_couple";
    Og_printer.printer_Array
      (fun ctx ppf (f, f0) ->
       if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
         else
       if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@,"
         else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@[<hv 2>(@,";
       Format.fprintf ppf "%,@[<hv 0>";
       Og_printer.printer_Float Og_precedence.Ck_tuple_item ppf f;
       Format.fprintf ppf "%,,@ ";
       Og_printer.printer_Float Og_precedence.Ck_tuple_item ppf f0;
       Format.fprintf ppf "%,@]";
       if ctx = Og_precedence.Ck_argument then
         Format.fprintf ppf "%,@;<0 -2>)" else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@;<0 -2>)@]")
      Og_precedence.Ck_argument
      ppf
      t_f_f_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Vfloat_triple t_f_f_f_a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Vfloat_triple";
    Og_printer.printer_Array
      (fun ctx ppf (f, f0, f1) ->
       if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
         else
       if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@,"
         else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@[<hv 2>(@,";
       Format.fprintf ppf "%,@[<hv 0>";
       Og_printer.printer_Float Og_precedence.Ck_tuple_item ppf f;
       Format.fprintf ppf "%,,@ ";
       Og_printer.printer_Float Og_precedence.Ck_tuple_item ppf f0;
       Format.fprintf ppf "%,,@ ";
       Og_printer.printer_Float Og_precedence.Ck_tuple_item ppf f1;
       Format.fprintf ppf "%,@]";
       if ctx = Og_precedence.Ck_argument then
         Format.fprintf ppf "%,@;<0 -2>)" else
       if ctx = Og_precedence.Ck_tuple_item then
         Format.fprintf ppf "%,@;<0 -2>)@]")
      Og_precedence.Ck_argument
      ppf
      t_f_f_f_a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_matrix ctx ppf = function
  | Typed_value.Mcint i_u_t ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Mcint";
    Umatrix_print.printer_t Og_printer.printer_Int
      Og_precedence.Ck_argument
      ppf
      i_u_t;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Mfint i_u_t ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Mfint";
    Umatrix_print.printer_t Og_printer.printer_Int
      Og_precedence.Ck_argument
      ppf
      i_u_t;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Mcfloat f_u_t ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Mcfloat";
    Umatrix_print.printer_t Og_printer.printer_Float
      Og_precedence.Ck_argument
      ppf
      f_u_t;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Typed_value.Mffloat f_u_t ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Typed_value.Mffloat";
    Umatrix_print.printer_t Og_printer.printer_Float
      Og_precedence.Ck_argument
      ppf
      f_u_t;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

(* Defining top printers. *)

let print_native_int ppf = printer_native_int Og_precedence.Ck_toplevel ppf

and print_int32 ppf = printer_int32 Og_precedence.Ck_toplevel ppf

and print_int64 ppf = printer_int64 Og_precedence.Ck_toplevel ppf
;;

let print_t ppf = printer_t Og_precedence.Ck_toplevel ppf

and print_scalar ppf = printer_scalar Og_precedence.Ck_toplevel ppf

and print_vector ppf = printer_vector Og_precedence.Ck_toplevel ppf

and print_matrix ppf = printer_matrix Og_precedence.Ck_toplevel ppf
;;

(* End of OCamlGen, 4.0+dev0 generated code. *)
