(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(*  This example is based on an example designed for OCamlP3l.         *)
(*                                                                     *)
(***********************************************************************)

(* $Id: main.ml,v 1.17 2017/01/05 16:00:22 fclement Exp $ *)

module Nat = Unatural;;
module Co = Ucommand_option;;

open Rdec2;;
open Options;;
open Make_dd2;;
open Make_skl2;;

let set_number_of_workers rdec =
  let (nsbdx, nsbdy) = number_of_subdomains_of_regular_decomposition rdec in
  Sklml_options.set_number_of_workers (Nat.mul nsbdx nsbdy);
;;

let generate_input_stream rdec = Sklstream.singleton (set_data rdec);;

let finalize = print_result;;

let main { Parfun.parfun = parfun; } =
  Co.parse ();
  let (cfg_fname, cfg, rdec) as gp = get_global_parameters () in
  set_number_of_workers rdec;
  let is = generate_input_stream rdec in
  let os = parfun (dd_skl gp) is in
  Sklstream.iter ignore os;
  finalize cfg cfg_fname;
;;

Parfun.pardo main;;
