(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(*  This example is based on an example designed for OCamlP3l.         *)
(*                                                                     *)
(***********************************************************************)

(* $Id: make_skl2.ml,v 1.4 2017/05/15 16:54:59 fclement Exp $ *)

module Nat = Unatural;;
module A = Uarray;;

open Skl;;

open Rdec2;;
open Make_skl1;;

(* The atomic skeletons - part 2. *)

let subdomain_worker_skl =
  Skl.make_skl (fun (cfg, subd) -> Make_dd2.subdomain_worker cfg subd)
;;

(* The full computational skeleton expression. *)
let dd_skl (cfg_fname, cfg, rdec) =
  let nw = Sklml_options.get_number_of_workers () in
  Printf.eprintf "cfg_fname = %s\nnworkers = %d\n" cfg_fname (Nat.to_int nw);
  let mk_worker i =
    (subdomain_worker_skl (cfg, { dec = rdec; number = i; }),
     i :: neighbors_of { dec = rdec; number = i; }) in
  let spec = A.init nw mk_worker in
  Make_domain.mk_domain_opt spec
    (plot_skl (cfg, rdec) ||| convergence_skl cfg)
;;
