(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: algo.ml,v 1.12 2017/01/04 11:58:31 fclement Exp $ *)

module Nat = Unatural;;
module L = Ulist;;

(* Begin Printing and reporting *)
let printer_Int_List = Og_printer.printer_List Og_printer.printer_Int;;

let print_ident_value ident printer ppf value =
  Format.fprintf ppf "%s =@ %a"
    ident
    (printer Og_precedence.Ck_toplevel) value
;;

let report ident printer ppf value =
  Format.fprintf ppf "@[<hv 2>%a@]@."
    (print_ident_value ident printer) value
;;

let report_int_list ident = report ident printer_Int_List;;

let ppf = Format.std_formatter;;

let report_state (i, li) =
  let sli = Printf.sprintf "l%i" (Nat.to_int i) in
  report_int_list sli ppf li;
;;
(* End Printing and reporting *)

type state = Nat.t * int list;;

let init () =
  let l0 = L.init Nat._4_ Nat.to_int in
  (Nat._0_, l0 : state)
;;

let imax = Nat._5_;;

let stop (i, _ : state) = i >= imax;;

let iteration f (i, li) = (Nat.succ i, 1 :: f li);;

let rec iterations f (st_i : state) =
  report_state st_i;
  if stop st_i then st_i else
  let st_ip1 = iteration f st_i in
  iterations f st_ip1
;;

let final = report_state;;

let algo f =
  let st_0 = init () in
  let st_n = iterations f st_0 in
  final st_n;
;;
