(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: main.ml,v 1.10 2017/05/15 13:58:10 fclement Exp $ *)

module Nat = Unatural;;

let _2_ = Nat._2_;;

let square i = i * i;;

let compute i = (square (succ i) - square (pred i)) / 2;;

(* Lift the sequential function compute to a skeleton. *)
(*let compute_skl = skl () -> compute;;*)
let compute_skl = Skl.make_skl (fun () -> compute);;

(* Define the required computation network for the compute function. *)
let parallel_compute parfun =
  Skl_extra.list_farm parfun (compute_skl ())
;;

let main { Parfun.parfun = parfun; } =
  let nw = _2_ in
  (* Define the parallel computation network for function compute *)
  let compute = parallel_compute parfun nw in
  Algo.algo compute;
  exit 0;
;;

Parfun.pardo main;;
