(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: model.ml,v 1.14 2017/01/04 11:58:31 fclement Exp $ *)

module Nat = Unatural;;
module L = Ulist;;
module Ar = Uaref;;
module D = Udebug;;

open Misc_sprint;;

let { Ar.get_flag = get_initialized; set_flag = set_initialized; _; } =
  Ar.make_flag ()
;;

let check_initialized tname =
  if not (get_initialized ()) then raise (Pio.Uninitialized_worker tname)
;;

(* Worker initialization. *)

let default_data = 0;;

let { Ar.get = get_data; set = set_data; } =
  Ar.make_aref_unique "worker data" default_data
;;

let init_data = 1;;

let initialization () =
  set_data init_data;
  set_initialized ();
;;

(* Init engine. *)

let init () = L.init Nat._4_ Nat.to_int;;

let init_engine _spf () =
  check_initialized "init";
  init ()
;;

(* Light comp engine. *)

let light_comp d i = i + d;;

let light_comp_engine { D.spf_debug = debug; _; } =
  check_initialized "light comp";
  let d = get_data () in
  fun i ->
    debug (sprint_int "i" i);
    light_comp d i
;;

(* Heavy comp engine. *)

let square i = i * i;;

let heavy_comp d i = (square (i + d) - square (i - d)) / 2;;

let heavy_comp_engine { D.spf_debug = debug; _; } =
  check_initialized "heavy comp";
  let d = get_data () in
  fun i ->
    debug (sprint_int "i" i);
    heavy_comp d i
;;

(* Final engine. *)

let final_engine _spf () = check_initialized "final";;
