(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: expor.mli,v 1.26 2017/01/04 11:58:27 fclement Exp $ *)

(** {3 Exportable closures handling}

 Closures that can be exported are the result of lambda-lifted [seq]
 enclosed expressions.
 Two schemes are used:
 - if expression [EXPR] has free parameters, we lift them in a tuple in
   order to keep a unified arity:
     [fun (a, b, c) -> EXPR (a, b, c)]
 - if expression [EXPR] is closed, we lift it using a dummy tuple,
   again keeping the same arity:
     [fun () -> EXPR]

 Thus, lifted [seq] enclosed expressions have the following type:
   ['something -> 'a -> 'b]
 Typically ['a] and ['b] are streams. *)

open Common_types;;

type ('x, 'a, 'b) t
and ('x, 'a, 'b) expor = ('x, 'a, 'b) t
;;

val register : ('x -> 'a Sklstream.t -> 'b Sklstream.t) -> ('x, 'a, 'b) t;;

val retreive : ('x, 'a, 'b) t -> 'x -> 'a Sklstream.t -> 'b Sklstream.t;;
val unsafe_retreive : Unatural.t -> 'x -> 'a Sklstream.t -> 'b Sklstream.t;;

val get_exportable : 'x * ('x, 'a, 'b) t -> Obj.t * atom_id;;

val is_registered :
  ('x -> 'a Sklstream.t -> 'b Sklstream.t) -> ('x, 'a, 'b) t option
;;
