(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: make_command.ml,v 1.17 2017/05/15 17:29:20 fclement Exp $ *)

module L = Ulist;;

module Mk_cmd = Tools_make_command;;

open Options;;

type compilation_phase =
  | Build_library
  | Compile_only
  | Link
;;

let compilation_phase opts =
  if L.mem "-a" opts then Build_library else
  if L.mem "-c" opts then Compile_only else
  Link
;;

let get_compiler_libs_and_include_dirs
    { compilation_mode = comp_mode; semantics_mode = sem_mode;
      lib_root_dir = dir; ocaml_options = opts; _; } =
  let (compiler, lib_suffix) =
    Sklml_path.get_compiler_and_lib_suffix comp_mode in
  let (libs, include_dirs, opts) =
    match compilation_phase opts with
    | Build_library -> ([], [], opts)
    | Compile_only ->
      ([], Sklml_path.sklml_compilation_include_dirs dir, opts)
    | Link ->
      Mk_cmd.add_system lib_suffix
        (Mk_cmd.add_pph dir lib_suffix
          (Mk_cmd.sklml_libs_and_include_dirs
            sem_mode dir opts lib_suffix)) in
  (compiler, libs, include_dirs, opts)
;;

let mk_command comp_opts =
  let (compiler, libs, include_dirs, opts) =
    get_compiler_libs_and_include_dirs comp_opts in
  let include_options = Mk_cmd.caml_include_options include_dirs in
  Mk_cmd.unwords ([ compiler; ] @ include_options @ libs @ opts)
;;
