(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: make_domain.mli,v 1.22 2017/01/04 11:58:28 fclement Exp $ *)

open Tools_common_types;;

type 'a border = worker_index * 'a
(** An ['a border] is the pair of an integer which uniquely identifies the
 worker who sent it and a data of type ['a] (the border data which is
 application specific). *)
;;

type 'a borders = 'a border list;;

type worker_connectivity = worker_index list
and connectivity_table = worker_connectivity array
;;

type ('a, 'b) worker_spec = ('a borders, 'a * 'b) Skl.t * worker_connectivity
(** An [('a, 'b) worker_spec] value is the description of a subdomain worker.
 The first component of the description is the worker's action (a skeleton),
 the second component is the worker connectivity.
 The connectivity tells which input is needed by the worker. *)
;;

val mk_domain :
  ('a, 'b) worker_spec array -> ('b array, bool) Skl.t ->
  ('a array, ('a * 'b) array) Skl.t
(** Builds a domain skeleton from a worker specification and a convergence
 predicate. *)
;;

val mk_f_domain :
  ('a, 'b) worker_spec array -> ('b array -> bool) ->
  ('a array, ('a * 'b) array) Skl.t
(** [mk_f_domain] is an optimizing version of [mk_domain] devoted to
 convergence predicate that are simple functions.
 Warning: you must use [mk_f_domain] at toplevel in your program, before
 calling [pardo], because [mk_f_domain] registers skeletons depending on its
 arguments. *)
;;

val mk_domain_opt :
  ('a, 'b) worker_spec array -> ('b array, bool) Skl.t ->
  ('a array, ('a * 'b) array ) Skl.t
(** Optimal version, currently unsafe. *)
;;
val mk_f_domain_opt :
  ('a, 'b) worker_spec array -> ('b array -> bool) ->
  ('a array, ('a * 'b) array) Skl.t
(** Optimal version, currently unsafe. *)
;;
