(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: skl_extra.ml,v 1.17 2017/01/04 11:58:28 fclement Exp $ *)

(* If you can build an interesting skeleton from existing ones, put it
   here *)

module A = Uarray;;

open Skl;;

let from_some = function
  | None -> invalid_arg "from_some"
  | Some x -> x
;;

(* Basic skeletons. *)

let dup =
  let f_dup x = (x, x) in
  let dup_pure () = Sklstream.map f_dup in
  ignore (Expor.register dup_pure);
  fun () -> atom ((), from_some (Expor.is_registered dup_pure))
;;

let projl =
  let f_projl = fst in
  let projl_pure () = Sklstream.map f_projl in
  ignore (Expor.register projl_pure);
  fun () -> atom ((), from_some (Expor.is_registered projl_pure))
;;

let projr =
  let f_projr = snd in
  let projr_pure () = Sklstream.map f_projr in
  ignore (Expor.register projr_pure);
  fun () -> atom ((), from_some (Expor.is_registered projr_pure))
;;

let injl =
  let f_injl x = Inl x in
  let injl_pure () = Sklstream.map f_injl in
  ignore (Expor.register injl_pure);
  fun () -> atom ((), from_some (Expor.is_registered injl_pure))
;;

let injr =
  let f_injr x = Inr x in
  let injr_pure () = Sklstream.map f_injr in
  ignore (Expor.register injr_pure);
  fun () -> atom ((), from_some (Expor.is_registered injr_pure))
;;

(* unit -> ('a * bool, ('a, 'a) sum) Skl.t *)
let to_sum =
  let f_to_sum (x, b) = if b then Inl x else Inr x in
  let to_sum_pure () = Sklstream.map f_to_sum in
  ignore (Expor.register to_sum_pure);
  fun () -> atom ((), from_some (Expor.is_registered to_sum_pure))
;;

(* Shortcuts. *)

let if_then_else (cond_skl, then_skl, else_skl) =
  dup () ||| (id () *** cond_skl) ||| to_sum () ||| (then_skl +++ else_skl)
;;

let rails_same (skl, n) = rails (A.make n skl);;

(* Regular functions. *)

let list_farm parfun sk nw l =
  let is = Sklstream.of_list l in
  let os = parfun (Skl.farm (sk, nw)) is in
  Sklstream.to_list os
;;

let array_farm_i parfun sk nw a =
  let is = Sklstream.of_vect a in
  let os = parfun (Skl.farm_i (sk, nw)) is in
  Sklstream.to_vect os
;;
