(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: skl_extra.mli,v 1.19 2017/01/04 11:58:28 fclement Exp $ *)

open Tools_common_types;;

(** {6 Basic skeletons} *)

val dup : unit -> ('a, 'a * 'a) Skl.t;;
val projl : unit -> ('a * 'b, 'a) Skl.t;;
val projr : unit -> ('a * 'b, 'b) Skl.t;;
val injl : unit -> ('a, ('a, 'b) Skl.sum) Skl.t;;
val injr : unit -> ('b, ('a, 'b) Skl.sum) Skl.t;;

(** {6 Shortcuts} *)

val if_then_else :
  ('a, bool) Skl.t * ('a, 'b) Skl.t * ('a, 'b) Skl.t -> ('a, 'b) Skl.t
;;
val rails_same :
  ('a, 'b) Skl.t * number_of_workers -> ('a array, 'b array) Skl.t
;;

(** {6 Regular functions} *)

val list_farm :
  (('a, 'b) Skl.t -> 'a Sklstream.t -> 'b Sklstream.t) ->
  ('a, 'b) Skl.t -> number_of_workers -> 'a list -> 'b list
(** [list_farm parfun skel n l] maps in parallel the function behind skeleton
 [skel] for each items of list [l].
 For that, a [farm] skeleton is automatically deployed. *)
;;

val array_farm_i :
  (('a, 'b) Skl.t -> 'a Sklstream.t -> 'b Sklstream.t) ->
  (index -> ('a, 'b) Skl.t) -> number_of_workers -> 'a array -> 'b array
(** Same as {!list_farm}, but for arrays, and the argument skeleton may be
 initialized with the index number. *)
;;
