(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: tools_make_command.ml,v 1.10 2017/05/15 17:29:20 fclement Exp $ *)

module L = Ulist;;
module S = Ustring;;

let unwords = S.concat " ";;

let add_suffix suf x = x ^ suf;;

let add_with_suffix suf l x = add_suffix suf x :: l;;

let list_trd_mem x (_, _, l) = L.mem x l;;
let list_trd_rem x (l1, l2, l3) = (l1, l2, L.find_all (( <> ) x) l3);;

let add_unique suf add accu x =
  let xsuf = add_suffix suf x in
  let accu =
    if list_trd_mem xsuf accu then list_trd_rem xsuf accu else
    accu in
  add suf accu x
;;

let system_add suf (ls, ids, opts) lib =
  (add_with_suffix suf ls lib, ids, opts)

and pph_add dir suf (ls, ids, opts) lib =
  (add_with_suffix suf ls lib,
   Sklml_path.pph_include_dir dir lib :: ids,
   opts)
;;

let add_system_laid suf = add_unique suf system_add
and add_pph_laid dir suf = add_unique suf (pph_add dir)
;;

let add_libs_and_include_dirs f libs laids =
  L.fold_left f laids (L.rev libs)
;;

let add_system suf =
  add_libs_and_include_dirs (add_system_laid suf) Sklml_path.system_libs

and add_pph dir suf =
  add_libs_and_include_dirs (add_pph_laid dir suf) Sklml_path.pph_libs
;;

let sklml_libs_and_include_dirs sem_mode dir opts suf =
  (L.map (add_suffix suf) (Sklml_path.sklml_libs sem_mode),
   Sklml_path.sklml_link_include_dirs dir sem_mode,
   opts)
;;

let include_options k pl = L.flatten (L.map (fun p -> [ k; p; ]) pl);;

let caml_include_options = include_options "-I";;
